@@ -46,28 +46,27 @@ writeC mod domain lds ctxt xs out copts
 -- the Read-Eval-Print Loop.
 writeout :: String -> String -> [Output] -> String
 writeout header mprog xs = header ++
-                           externC (writedecls xs) ++
-		           writeout' xs ++ mprog
+                           externC (writedecls xs)
+		                       (writeout' xs ++ mprog)
 
 writeout' [] = ""
 writeout' ((RawOutput str):xs) = str ++ "\n" ++ writeout' xs
 writeout' ((ExternOutput (f, ty)):xs) 
-    = externC $ "void " ++ (show f) ++ mangling ty ++ "(VMState* vm);\n" 
-				  ++ writeout' xs
+    = externC ("void " ++ (show f) ++ mangling ty ++ "(VMState* vm);\n")
+				  (writeout' xs)
 writeout' ((ExcOutput nm defhere):xs)
-    = externC $
+    = externC (
          (if defhere then "" else "extern ") ++
             "const char* " ++ show nm ++ 
          (if defhere then " = \"" ++ showuser nm ++"\"" else "") ++
-            ";\n" ++ writeout' xs
+            ";\n" )  (writeout' xs)
 writeout' ((FNOutput (f, ty, pop, args, def)):rest) 
-    = "void " ++ (show f) ++ mangling ty ++ "(VMState* vm){\n" ++ pop ++ "\n" ++ def ++ "}\n\n"
+    = externC ("void " ++ (show f) ++ mangling ty ++ "(VMState* vm){\n" ++ pop ++ "\n" ++ def ++ "}\n\n") ""
       ++ writeout' rest 
 writeout' ((GlobOutput n):rest)
-    = externC $ "DECLGLOBAL("++show n++");\n"
-      ++ writeout' rest
+    = externC ("DECLGLOBAL("++show n++");\n") (writeout' rest)
 
-externC x = "\nextern \"C\" {\n" ++ x ++ "\n}\n"
+externC x rest = "\nextern \"C\" {\n" ++ x ++ "\n}\n" ++ rest
 
 writedecls [] = ""
 writedecls ((FNOutput (f, ty, _, _, _)):rest) 
@@ -207,7 +206,7 @@ instr MULTOP = "MULTOP"
 instr DIVTOP = "DIVTOP"
 instr APPENDTOP = "APPENDTOP"
 instr APPENDTOPINT = "APPENDTOPINT"
-instr (MKARRAY i) = "MKARRAY("++show i++")"
+instr (MKARRAY s i) = "MKARRAY2("++show (fst s)++","++show i++")"
 instr (TMPSET tmp val) = "t"++show tmp++"="++show val
 instr (RTMPSET tmp val) = "t"++show tmp++"="++show val
 instr (CALL v) = "CALLFUN("++show (fst v)++")"
@@ -220,10 +219,13 @@ instr (FASTCALL nm args) = nm ++ "(vm, " ++ showlist args ++ ")"
 instr (TAILCALL v) = "TAILCALLFUN("++show (fst v)++")"
 instr (TAILCALLNAME f) = "TAILCALL("++f++")"
 instr TAILCALLTOP = "TAILCALLTOP"
-instr (CLOSURE n i arity) 
-    = "CLOSURE("++ n ++","++show i++"," ++ show arity ++")"
-instr (CLOSURELOC v i) = "CLOSURELOC("++ show (fst v) ++","++show i++")"
-instr (CLOSURETOP i) = "CLOSURETOP("++show i++")"
+instr (CLOSURE b n i arity) 
+    = "CLOSURE("++ n ++","++show i++"," ++ show arity ++"," ++ 
+      (if b then "1)" else "0)")
+instr (CLOSURELOC b v i) = "CLOSURELOC("++ show (fst v) ++","++show i++","++
+                           (if b then "1)" else "0)")
+instr (CLOSURETOP b i) = "CLOSURETOP("++show i++"," ++
+      (if b then "1)" else "0)")
 instr (FOREIGNCALL n lib ty args) = mkfcall n lib ty args 
 instr (MKCON t i) = "MKCON("++show t ++ "," ++ show i ++ ")"
 instr (MKCONZERO t) = "MKCONZERO("++show t ++ ")"
@@ -306,6 +308,7 @@ instr (STACKTMP var v) = "STACKINT("++show (fst var)++",t"++show v++")";
 instr (STACKREAL var (RVAL v)) = "STACKREAL("++show (fst var)++","++show v++")";
 instr (STACKREAL var (REAL t)) = "STACKREAL("++show (fst var)++",t"++show t++")";
 instr (STACKSTR var (STR v)) = "STACKSTR("++show (fst var)++",L"++show v++")";
+instr (STACKSTR var (EMPTYSTR)) = "STACKSTR("++show (fst var)++",L\"\")";
 instr (TMPSETTOP t) = "TMPSETTOP(t"++show t++")"
 instr (PUSHSETTOP x) = "PUSHSETTOP("++pushitem x++")"
 instr (SETLOOKUP i v) = "SETLOOKUP("++show i ++", " ++ show (fst v)++")"
@@ -387,10 +390,10 @@ mkfcall n lib ty args = popvals 0 (length args) ++
 {- CIM 12/7/05 changed to KVOID to avoid clashes with MinGW -}
     conv (Prim Void) = "KVOID("
     conv (Prim Number) = "DECLAREQUICK(fr);STACKINT(fr,"
-    conv (Prim RealNum) = "PUSH(MKREAL"
+    conv (Prim RealNum) = "DECLAREQUICK(fr);STACKREAL(fr,"
     conv (Prim Boolean) = "DECLAREQUICK(fr);STACKINT(fr,"
     conv (Prim Character) = "DECLAREQUICK(fr);STACKINT(fr,(kint)"
-    conv (Prim StringType) = "PUSH(MKSTR"
+    conv (Prim StringType) = "DECLAREQUICK(fr);STACKSTR(fr,"
     conv (Prim Pointer) = "DECLAREQUICK(fr);STACKRAW(fr,"
     conv (TyVar _) = "PUSH(" -- Enough rope to hang yourself with!
     conv (Array _) = "PUSH(MKARRAYVAL"
