@@ -146,7 +146,7 @@ void visibility(int state)
     KayaCall(visibilityfun,2,KayaInt(state),visibilitydata);
 }
 
-int _glut_getMode(KayaValue k)
+kint _glut_getMode(KayaValue k)
 {
     UTAG(k) {
     case 0: return GLUT_RGB;
@@ -170,5 +170,5 @@ void doGlutInit(KayaArray modes)
     char *fake_argv[1];
     fake_argv[0]=(char*)"test";
     glutInit(&fake_argc, fake_argv) ;
-    glutInitDisplayMode(orArray(modes,_glut_getMode));
+    glutInitDisplayMode((int)orArray(modes,_glut_getMode));
 }
