@@ -25,9 +25,9 @@ class PGRes : public gc
 {
 public:
     KayaArray res_table;
+    KayaArray col_names;
     int rows;
     int cols;
-    PGresult* pgres;
 };
 
 extern "C" {
@@ -39,14 +39,14 @@ extern "C" {
     wchar_t* pg_getError(void* conn);
 /// Returns a PGRes
     void* pg_exec(void* vmptr,void* conn,wchar_t* query, KayaValue err);
-/// Returns a column name
-    wchar_t* pg_columnname(void* vmptr,void* res, int col);
 /// Gets the strings from a PGRes
     KayaArray pg_getstrs(void* res);
 /// Return the number of rows in a result
     int pg_numrows(void* res);
 /// Return the number of columns in a result
     int pg_numcols(void* res);
+/// Returns the field names in a result
+    Array* pg_colnames(void* res);
 /// Close the connection
     void pg_close(void* conn);
     void* do_pg_prepare(wchar_t* query);
