@@ -11,6 +11,10 @@
 
 using namespace std;
 
+#ifndef MEM_POOLSIZE
+#define MEM_POOLSIZE 1
+#endif
+
 #ifndef WIN32
 #define SWPRINTF swprintf
 #else
@@ -42,8 +46,15 @@ class String;
 class Array;
 class Exception;
 class Union;
+class Real;
 class Closure;
 
+class ValuePool;
+class StringPool;
+class UnionPool;
+class RealPool;
+class ArrayPool;
+
 #define KMALLOC(x) GC_MALLOC(x)
 #define KMALLOC_BLOCK(x) GC_MALLOC_ATOMIC(x)
 #define KMALLOC_ROOT(x) GC_MALLOC_UNCOLLECTABLE(x)
@@ -52,11 +63,91 @@ class Closure;
 #define NEWVAL (new Value(NULL,KVT_INT))
 //(Value*)(KMALLOC(sizeof(Value))))
 #define MKVAL(ptr,ft) new Value(ptr,ft)
+#define MKINITVAL(ptr,ft) new(GCInit) Value(ptr,ft)
+#define MKPVAL(place,ptr,ft) new(place) Value(ptr,ft)
 //NEWVAL->setPtr(ptr,ft)
 
 #define NEWREAL(n) new Real(n)
+#define NEWPREAL(p,n) new(p) Real(n)
 //(((Real*)(KMALLOC(sizeof(Real))))->setNum(n))
 
+
+// If we're allocating globals, we'd better do a GC_init first. Use this
+// as a placement to the 'new' operator on Values.
+enum DoInit { GCInit, NoGCInit };
+
+
+class ValuePool {
+public:
+  void *operator new(size_t num_bytes) {
+    return (KMALLOC_ROOT(num_bytes));
+  }
+
+  ValuePool();
+  void* next();
+private:
+  Value* m_pool;
+  int m_usage;
+  void refill();
+};
+
+class StringPool {
+public:
+  void *operator new(size_t num_bytes) {
+    return (KMALLOC_ROOT(num_bytes));
+  }
+
+  StringPool();
+  void* next();
+private:
+  String* m_pool;
+  int m_usage;
+  void refill();
+};
+
+class UnionPool {
+public:
+  void *operator new(size_t num_bytes) {
+    return (KMALLOC_ROOT(num_bytes));
+  }
+
+  UnionPool();
+  void* next();
+private:
+  Union* m_pool;
+  int m_usage;
+  void refill();
+};
+
+class RealPool {
+public:
+  void *operator new(size_t num_bytes) {
+    return (KMALLOC_ROOT(num_bytes));
+  }
+
+  RealPool();
+  void* next();
+private:
+  Real* m_pool;
+  int m_usage;
+  void refill();
+};
+
+class ArrayPool {
+public:
+  void *operator new(size_t num_bytes) {
+    return (KMALLOC_ROOT(num_bytes));
+  }
+
+  ArrayPool();
+  void* next();
+private:
+  Array* m_pool;
+  int m_usage;
+  void refill();
+};
+
+
 // I really shouldn't call this Real! Float/Double better.
 class Real
 {
@@ -64,6 +155,14 @@ public:
     void *operator new(size_t num_bytes) {
 	return (KMALLOC(num_bytes));
     }
+    void *operator new(size_t num_bytes, void* place) {
+      return place;
+    }
+  // create in preallocated memory
+    void *operator new(size_t num_bytes, RealPool* allocator) {
+      return allocator->next();
+    }
+
 
     Real(double n):number(n) {}
 //    Real* setNum(double n) { number = n; return this; }
@@ -87,15 +186,23 @@ class Union : public gc_cleanup
 {
 public:
     // If getargs is true, get arguments from the stack in vm.
-    Union(VMState* vm, unsigned arity, bool getargs = true);
+  // create in preallocated memory
+    void *operator new(size_t num_bytes, UnionPool* allocator) {
+      return allocator->next();
+    }
+    void *operator new(size_t num_bytes) {
+      return (void*)GC_MALLOC(num_bytes);
+    }
+
+    Union(VMState* vm, ukint arity, bool getargs = true);
     ~Union() { if (args != NULL) { GC_FREE(args); } }
 
     bool eq(Union* x, map<Value*,Value*>& done);
-    int cmp(Union* x);
-    unsigned memusage();
-    Union* copy(map<Value*,Value*>& done);
+    kint cmp(Union* x);
+    ukint memusage();
+    Union* copy(VMState* vm, map<Value*,Value*>& done);
 
-    unsigned arity; 
+    ukint arity; 
     Value** args;
 };
 
@@ -133,24 +240,27 @@ public:
     void *operator new(size_t num_bytes) {
 	return (KMALLOC(num_bytes));
     }
+    void *operator new(size_t num_bytes, void* place) {
+      return place;
+    }
 
     Exception(VMState* vm);
-    Exception(VMState* vm, const char* tag, int arity);
+    Exception(VMState* vm, const char* tag, kint arity);
     void show();
     bool eq(Exception* x);
-    int cmp(Exception* x);
+    kint cmp(Exception* x);
 
     void dumpBacktrace();
     
     String* err;
-    int code;
+    kint code;
 
     const char* tag;
-    int arity;
+    kint arity;
     Value** args;
 
     wchar_t* throwfile;
-    int throwline;
+    kint throwline;
 
     // Need a copy of entire backtrace. Ick.
     vector<CallStackEntry> backtrace;
@@ -165,6 +275,22 @@ public:
     void *operator new(size_t num_bytes) {
 	return (KMALLOC(num_bytes));
     }
+  // create in preallocated memory
+    void *operator new(size_t num_bytes, void* allocptr) {
+      return allocptr;
+    }
+  // create in preallocated memory
+    void *operator new(size_t num_bytes, ValuePool* allocator) {
+      return allocator->next();
+    }
+ // Create a global, better do initialisation first
+    void *operator new(size_t num_bytes, DoInit init) {
+	if (init==GCInit) {
+	    GC_init();
+	}
+	return KMALLOC(num_bytes);
+    }
+
 
     valtype getType() { return m_funtable; }
 
@@ -172,6 +298,9 @@ public:
     Value* clone() {
 	return MKVAL(m_val,m_funtable);
     }
+    Value* cloneTo(ValuePool* valpool) {
+	return MKPVAL(valpool,m_val,m_funtable);
+    }
 
     /// Get an integer out. Assumes you've checked/know.
     kint getInt() { return (kint)m_val; };
@@ -205,21 +334,27 @@ public:
     void runClosure(VMState *vm);
 
     /// Add more arguments from the stack to the closure stored here.
-    Value* addClosureArgs(VMState *vm, int i);
+    Value* addClosureArgs(VMState *vm, kint i, bool lam);
 
     /// Coercions
     void int2str();
+    void int2str(StringPool*);
     void str2int();
     void str2real();
+    void str2real(RealPool*);
     void chr2str();
+    void chr2str(StringPool*);
     void bool2str();
+    void bool2str(StringPool*);
     void str2chr();
     void int2real();
+    void int2real(RealPool*);
     void real2int();
     void real2str();
+    void real2str(StringPool*);
 
     /// Get the tag out of a union
-    int getTag() { return m_funtable-KVT_UNION; }
+    kint getTag() { return m_funtable-KVT_UNION; }
 
 
     /// Get the raw pointer out.
@@ -241,6 +376,7 @@ public:
 
     /// Update to be a real
     void setReal(double i);
+    void setReal(RealPool* r, double v);
 
     /// Update to be a function.
     void setFunc(Closure* f);
@@ -289,21 +425,21 @@ public:
     }
 
     /// Set a value from the projection of the top stack item
-    void project(VMState* vm, int i);
-    void fastproject(Value* topitem, int i) {
+    void project(VMState* vm, kint i);
+    void fastproject(Value* topitem, kint i) {
 	Union* top=(Union*)(topitem->m_val);
 	Value* arg=top->args[i];
 	m_val=arg->m_val;
 	m_funtable = arg->m_funtable;
     }
-    void exceptionproject(Value* topitem, int i) {
+    void exceptionproject(Value* topitem, kint i) {
 	Exception* top=(Exception*)(topitem->m_val);
 	Value* arg=top->args[i];
 	m_val=arg->m_val;
 	m_funtable = arg->m_funtable;
     }
 
-    void fastproject(VMState* vm, int i);
+    void fastproject(VMState* vm, kint i);
 
     /// Lookup a value in an array --- resize/create if necessary.
     Value* lookup(VMState* vm, kint i);
@@ -326,6 +462,7 @@ private:
 #define OFFSET_LIMIT 8192
 
 #define NEWSTRING(x) new String(x)
+#define NEWPSTRING(p,x) new(p) String(x)
 //(((String*)(KMALLOC(sizeof(String))))->setVal(x))
 
 class String
@@ -334,10 +471,18 @@ public:
     String(const wchar_t* val);
     String(wchar_t val);
     String(kint len);
+    String(VMState* vm);
 
     void *operator new(size_t num_bytes) {
 	return (KMALLOC(num_bytes));
     }
+    void *operator new(size_t num_bytes, void* place) {
+      return place;
+    }
+  // create in preallocated memory
+    void *operator new(size_t num_bytes, StringPool* allocator) {
+      return allocator->next();
+    }
 
     String* setVal(const wchar_t* val);
     String* setVal(wchar_t val);
@@ -353,17 +498,17 @@ public:
     void append(String* s);
     void append(wchar_t c);
     void append(const wchar_t* s);
+    void getForeign(const char* c);
     bool eq(String* s);
     bool eq(const wchar_t* s);
-    int cmp(String* s);
+    kint cmp(String* s);
     // Increase the offset. If it goes over OFFSET_LIMIT, reallocate
     void offset(kint x);
     // Chop last x characters off.
     void chop(kint x);
-  //    int hash();
 
   // debug/marshalling purposes
-    int space() { return M_ALLOC; }
+    kint space() { return M_ALLOC; }
 
 private:
     wchar_t* m_str;
@@ -374,4 +519,9 @@ private:
     kint m_len;
 };
 
+
+
+
+
+
 #endif
