@@ -37,66 +37,70 @@ extern "C" {
     kint inttable_eq(Value*, Value*, map<Value*,Value*>& done);
     kint inttable_fasteq(Value*, Value*);
     kint inttable_cmp(Value*, Value*);
-    Value* inttable_copy(Value*, map<Value*,Value*>& done);
-    Value* inttable_fastcopy(Value*);
+    Value* inttable_fastcopy(VMState*,Value*);
     void inttable_marshal(VMState*, String*, vector<Value*>&, Value*, kint id);
     Value* inttable_reflect(VMState*, map<Value*, Value*>&, Value*);
     int inttable_memusage(Value*);
+    kint inttable_hash(Value*);
 
     kint fntable_eq(Value*, Value*, map<Value*,Value*>& done);
     kint fntable_cmp(Value*, Value*);
-    Value* fntable_copy(Value*, map<Value*,Value*>& done);
+    Value* fntable_copy(VMState*, Value*, map<Value*,Value*>& done);
     void fntable_marshal(VMState*, String*, vector<Value*>&, Value*, kint id);
     Value* fntable_reflect(VMState*, map<Value*, Value*>&, Value*);
     int fntable_memusage(Value*);
+    kint fntable_hash(Value*,vector<Value*>&);
 
     kint realtable_eq(Value*, Value*, map<Value*,Value*>& done);
     kint realtable_fasteq(Value*, Value*);
     kint realtable_cmp(Value*, Value*);
-    Value* realtable_copy(Value*, map<Value*,Value*>& done);
-    Value* realtable_fastcopy(Value*);
+    Value* realtable_fastcopy(VMState*,Value*);
     void realtable_marshal(VMState*, String*, vector<Value*>&, Value*, kint id);
     Value* realtable_reflect(VMState*, map<Value*, Value*>&, Value*);
     int realtable_memusage(Value*);
+    kint realtable_hash(Value*);
 
     kint stringtable_eq(Value*, Value*, map<Value*,Value*>& done);
     kint stringtable_fasteq(Value*, Value*);
     kint stringtable_cmp(Value*, Value*);
-    Value* stringtable_copy(Value*, map<Value*,Value*>& done);
-    Value* stringtable_fastcopy(Value*);
+    Value* stringtable_fastcopy(VMState*,Value*);
     void stringtable_marshal(VMState*, String*, vector<Value*>&, Value*, kint id);
     Value* stringtable_reflect(VMState*, map<Value*, Value*>&, Value*);
     int stringtable_memusage(Value*);
+    kint stringtable_hash(Value*);
 
     kint arraytable_eq(Value*, Value*, map<Value*,Value*>& done);
     kint arraytable_cmp(Value*, Value*);
-    Value* arraytable_copy(Value*, map<Value*,Value*>& done);
+    Value* arraytable_copy(VMState*, Value*, map<Value*,Value*>& done);
     void arraytable_marshal(VMState*, String*, vector<Value*>&, Value*, kint id);
     Value* arraytable_reflect(VMState*, map<Value*, Value*>&, Value*);
     int arraytable_memusage(Value*);
+    kint arraytable_hash(Value*,vector<Value*>&);
 
     kint uniontable_eq(Value*, Value*, map<Value*,Value*>& done);
     kint uniontable_cmp(Value*, Value*);
-    Value* uniontable_copy(Value*, map<Value*,Value*>& done);
+    Value* uniontable_copy(VMState*, Value*, map<Value*,Value*>& done);
     void uniontable_marshal(VMState*, String*, vector<Value*>&, Value*, kint id);
     Value* uniontable_reflect(VMState*, map<Value*, Value*>&, Value*);
     int uniontable_memusage(Value*);
+    kint uniontable_hash(Value*,vector<Value*>&);
 
     kint cuniontable_eq(Value*, Value*, map<Value*,Value*>& done);
     kint cuniontable_cmp(Value*, Value*);
-    Value* cuniontable_copy(Value*, map<Value*,Value*>& done);
+    Value* cuniontable_copy(VMState*, Value*, map<Value*,Value*>& done);
     void cuniontable_marshal(VMState*, String*, vector<Value*>&, Value*, kint id);
     Value* cuniontable_reflect(VMState*, map<Value*, Value*>&, Value*);
     int cuniontable_memusage(Value*);
+    kint cuniontable_hash(Value*);
 
     kint exceptiontable_eq(Value*, Value*, map<Value*,Value*>& done);
     kint exceptiontable_fasteq(Value*, Value*);
     kint exceptiontable_cmp(Value*, Value*);
-    Value* exceptiontable_copy(Value*, map<Value*,Value*>& done);
     Value* exceptiontable_fastcopy(Value*);
     void exceptiontable_marshal(VMState*, String*, vector<Value*>&, Value*, kint id);
     Value* exceptiontable_reflect(VMState*, map<Value*, Value*>&, Value*);
     int exceptiontable_memusage(Value*);
+    kint exceptiontable_hash(Value*);
 }
 
 #endif
