@@ -11,17 +11,16 @@
 #include "wchar.h"
 #include "Heap.h"
 
-Value* unmarshal(void* vmptr, wchar_t* x, int id);
+Value* unmarshal(void* vmptr, wchar_t* x, kint id);
 
-Value* unpickle(VMState* vm, vector<Value*>&,wchar_t* &x,int id);
-int read_int(VMState* vm, wchar_t* &x);
+Value* unpickle(VMState* vm, vector<Value*>&,wchar_t* &x,kint id);
+kint read_int(VMState* vm, wchar_t* &x);
 Value* un_int(VMState* vm, vector<Value*>&,wchar_t* &x);
 Value* un_real(VMState* vm, vector<Value*>&,wchar_t* &x);
 Value* un_circle(VMState* vm, vector<Value*>&,wchar_t* &x);
 Value* un_string(VMState* vm, vector<Value*>&,wchar_t* &x);
-Value* un_array(VMState* vm, vector<Value*>&,wchar_t* &x,int id);
-Value* un_union(VMState* vm, vector<Value*>&,wchar_t* &x,int id);
-Value* un_closure(VMState* vm, vector<Value*>&,wchar_t* &x,int id);
-Union* un_union_static(int tag);
+Value* un_array(VMState* vm, vector<Value*>&,wchar_t* &x,kint id);
+Value* un_union(VMState* vm, vector<Value*>&,wchar_t* &x,kint id);
+Value* un_closure(VMState* vm, vector<Value*>&,wchar_t* &x,kint id);
 
 #endif
