@@ -21,8 +21,8 @@ extern "C" {
     KayaArray getBlockData(void* block, int offset,int size);
     void setBlockData(void* block, int offset, int size, KayaArray data);
 
-    int writeBlock(FILE* f,void* block, int size);
-    void* readBlock(FILE* f, KayaValue size);
+    int writeBlock(void* fptr,void* block, int size);
+    void* readBlock(void* fptr, KayaValue size);
 
     wchar_t* b64binary(void* block, int len);
     void* b64binarydec(wchar_t* block, KayaValue len);
