@@ -26,7 +26,7 @@ extern "C" {
   void do_gnutls_put(void* tls, wchar_t* rawdat);
   void do_gnutls_putbin(void* tls, void* rawdat, kint len);
   void do_gnutls_putbyte(void* tls, kint byte);
-  wchar_t* do_gnutls_get(void* tls, void* vmptr);
+  wchar_t* do_gnutls_get(void* tls, void* vmptr, kint rlen);
   void* do_gnutls_getbin(void* tls, void* vmptr, KValue len);
   kint do_gnutls_getbyte(void* tls, void* vmptr);
   void do_gnutls_close(void* tls, void* cred);
